/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;


/**
 * <p>
 * A utility class providing some common operations related to configuration. This class is declared as public for the
 * sub-packages to access. It is not intended for public use however.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class ConfigUtil {
    /**
     * <p>
     * Private constructor to prevent outside instantiation.
     * </p>
     */
    private ConfigUtil() {
    }

    /**
     * <p>
     * Obtains the root property of the configuration file in the specified namespace. This method conveniently wraps
     * the exceptions from the Configuration Manager into PersistenceConfigurationException.
     * </p>
     *
     * @param namespace the configuration namespace to use.
     *
     * @return the root property of the configuration file.
     *
     * @throws PersistenceConfigurationException if a problem occurs while configuring.
     */
    public static Property getRootProperty(String namespace)
        throws PersistenceConfigurationException {
        try {
            return ConfigManager.getInstance().getPropertyObject(namespace, "");
        } catch (ConfigManagerException e) {
            throw new PersistenceConfigurationException("Fails to load from configuration namespace " + namespace, e);
        }
    }

    /**
     * <p>
     * Gets the property value of the specified name. If the property value does not exist or is empty, a
     * PersistenceConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name.
     *
     * @throws PersistenceConfigurationException if the property value does not exist or is empty.
     */
    public static String getPropertyValue(Property property, String name)
        throws PersistenceConfigurationException {
        String value = getOptionalPropertyValue(property, name);

        if (value == null) {
            throw new PersistenceConfigurationException("Missing value for property " + name);
        }
        return value;
    }

    /**
     * <p>
     * Gets the int property value of the specified name. If the property value does not exist or is not an integer, a
     * PersistenceConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the int property value of the specified name.
     *
     * @throws PersistenceConfigurationException if the property value does not exist or is not an integer.
     */
    public static int getIntPropertyValue(Property property, String name)
        throws PersistenceConfigurationException {
        try {
            return Integer.parseInt(getPropertyValue(property, name));
        } catch (NumberFormatException e) {
            throw new PersistenceConfigurationException("Value for property " + name + " is not an integer");
        }
    }

    /**
     * <p>
     * Gets the optional property value of the specified name. If the property value exists but is empty, a
     * PersistenceConfigurationException will be thrown.
     * </p>
     *
     * @param property the property to get the value from.
     * @param name the property name.
     *
     * @return the property value of the specified name, or null if there is none.
     *
     * @throws PersistenceConfigurationException if the property value exists but is empty.
     */
    public static String getOptionalPropertyValue(Property property, String name)
        throws PersistenceConfigurationException {
        String value = property.getValue(name);

        if ((value != null) && (value.trim().length() == 0)) {
            throw new PersistenceConfigurationException("Empty value for property " + name);
        }
        return value;
    }
}
