/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.Util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import java.util.HashSet;
import java.util.Set;


/**
 * <p>
 * A class providing some common operations related to database persistence of collection index. This class assumes
 * that the DOCUMENT_COLLECTION and DOCUMENT_COLLECTION_DOCUMENT_XREF tables are common to the database persistence.
 * </p>
 *
 * <p>
 * This class and all its methods are package-private, so that they can only be used by classes in the same package. We
 * do not put the methods inside AbstractDBIndexPersistence because it is not required to have such table structures.
 * All methods assume valid arguments.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
class CollectionIndexPersistence {
    /**
     * <p>
     * The SQL statement to insert a collection.
     * </p>
     */
    private static final String SQL_INS_COLLECTION = "INSERT INTO DOCUMENT_COLLECTION VALUES (?)";

    /**
     * <p>
     * The SQL statement to insert a document to the collection.
     * </p>
     */
    private static final String SQL_INS_DOCUMENT = "INSERT INTO DOCUMENT_COLLECTION_DOCUMENT_XREF " + "VALUES (?, ?)";

    /**
     * <p>
     * The SQL statement to delete a collection.
     * </p>
     */
    private static final String SQL_DEL_COLLECTION = "DELETE FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID = ?";

    /**
     * <p>
     * The SQL statement to delete a document belonging to a collection.
     * </p>
     */
    private static final String SQL_DEL_DOCUMENT = "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF "
        + "WHERE COLLECTION_ID = ? AND DOCUMENT_ID = ?";

    /**
     * <p>
     * The SQL statement to delete the documents belonging to a collection.
     * </p>
     */
    private static final String SQL_DEL_DOCUMENTS = "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF "
        + "WHERE COLLECTION_ID = ?";

    /**
     * <p>
     * The SQL statement to select a collection.
     * </p>
     */
    private static final String SQL_SEL_COLLECTION = "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION "
        + "WHERE COLLECTION_ID = ?";

    /**
     * <p>
     * The SQL statement to select the document ids belonging to a collection.
     * </p>
     */
    private static final String SQL_SEL_DOCUMENT_IDS = "SELECT DOCUMENT_ID FROM DOCUMENT_COLLECTION_DOCUMENT_XREF "
        + "WHERE COLLECTION_ID = ?";

    /**
     * <p>
     * The SQL statement to select the documents belonging to a collection.
     * </p>
     */
    private static final String SQL_SEL_DOCUMENTS = "SELECT DOCUMENT.DOCUMENT_ID, LOCALE, SOURCE_IDENTITY, CLASS_NAME "
        + "FROM DOCUMENT_COLLECTION_DOCUMENT_XREF, DOCUMENT "
        + "WHERE COLLECTION_ID = ? AND DOCUMENT_COLLECTION_DOCUMENT_XREF.DOCUMENT_ID = DOCUMENT.DOCUMENT_ID";

    /**
     * <p>
     * The actual database persistence implementation to access the database. This is initialized in the constructor
     * and cannot be changed. Cannot be null.
     * </p>
     */
    private final AbstractDBIndexPersistence persistence;

    /**
     * <p>
     * Constructor that accepts the actual database persistence implementation.
     * </p>
     *
     * @param persistence the actual database persistence implementation.
     */
    CollectionIndexPersistence(AbstractDBIndexPersistence persistence) {
        this.persistence = persistence;
    }

    /**
     * <p>
     * Adds the collection index to the persistence. When the collection index exists in the persistence, it fails.
     * When the document indexes of the collection index are not persisted, it also fails.
     * </p>
     *
     * <p>
     * This method adds records to the DOCUMENT_COLLECTION and DOCUMENT_COLLECTION_DOCUMENT_XREF tables. The use count
     * is not updated here, since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionIndex the collection index to be persisted.
     *
     * @throws IndexPersistenceException if persisting the collection index fails; or the collection index exists in
     *         the persistence; or the document indexes of the collection index are not persisted.
     */
    void addCollectionIndex(CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        // get the collection id from the collection index
        String collectionId = collectionIndex.getId();

        Connection connection = persistence.getConnection();

        try {
            // start the transaction
            persistence.beginTransaction(connection);

            // add the collection to the DOCUMENT_COLLECTION table
            DBUtil.execute(connection, SQL_INS_COLLECTION, collectionId);

            // extract the document ids from the collection index
            Set documentIds = Util.getDocumentIds(collectionIndex.getAllDocumentIds());

            // add the documents to the DOCUMENT_COLLECTION_DOCUMENT_XREF table
            DBUtil.execute(connection, SQL_INS_DOCUMENT, collectionId, (String[]) documentIds.toArray(new String[0]));

            // commit the transaction
            persistence.commitTransaction(connection);
        } catch (IndexPersistenceException e) {
            persistence.rollbackTransaction(connection);
            throw e;
        } catch (SQLException e) {
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException("Fails to add collection index", e);
        }
    }

    /**
     * <p>
     * Removes the collection index from the persistence. The ID of the collection index is given. When the collection
     * index is not persisted, it fails.
     * </p>
     *
     * <p>
     * This method deletes records from the DOCUMENT_COLLECTION and DOCUMENT_COLLECTION_DOCUMENT_XREF tables. The use
     * count is not updated here, since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionId the ID of the collection index to be deleted.
     *
     * @throws IndexPersistenceException if deleting the collection index fails; or the collection index ID cannot be
     *         found in the persistence.
     */
    void removeCollectionIndex(String collectionId) throws IndexPersistenceException {
        Connection connection = persistence.getConnection();

        try {
            // start the transaction
            persistence.beginTransaction(connection);

            // check if the collection index exists
            if (!existCollectionIndex(connection, collectionId)) {
                throw new IndexPersistenceException("Collection index does not exist");
            }

            // remove the documents from the DOCUMENT_COLLECTION_DOCUMENT_XREF table
            DBUtil.execute(connection, SQL_DEL_DOCUMENTS, collectionId);

            // remove the collection from the DOCUMENT_COLLECTION table
            DBUtil.execute(connection, SQL_DEL_COLLECTION, collectionId);

            // commit the transaction
            persistence.commitTransaction(connection);
        } catch (IndexPersistenceException e) {
            persistence.rollbackTransaction(connection);
            throw e;
        } catch (SQLException e) {
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException("Fails to remove collection index", e);
        }
    }

    /**
     * <p>
     * Returns whether the collection index exists in the persistence. This method should be called prior to the
     * removal, update and retrieval of collection index.
     * </p>
     *
     * <p>
     * This method retrieves a record from the DOCUMENT_COLLECTION table.
     * </p>
     *
     * @param connection the connection to the database.
     * @param collectionId the ID of the collection whose index is checked.
     *
     * @return whether the collection index exists.
     *
     * @throws IndexPersistenceException if checking the collection index fails.
     */
    boolean existCollectionIndex(Connection connection, String collectionId)
        throws IndexPersistenceException {
        try {
            // check the collection in the DOCUMENT_COLLECTION table
            return DBUtil.exist(connection, SQL_SEL_COLLECTION, collectionId);
        } catch (SQLException e) {
            throw new IndexPersistenceException("Fails to check collection index", e);
        }
    }

    /**
     * <p>
     * Gets a set of all IDs of document indexes belonging to the collection. The ID of the collection index is given.
     * This method should be called prior to the retrieval of collection index.
     * </p>
     *
     * <p>
     * Normally the caller should have already checked whether the collection index exists in the persistence.
     * </p>
     *
     * @param connection the connection to the database.
     * @param collectionId the ID of the collection index.
     *
     * @return a set of WordSourceId instances.
     *
     * @throws IndexPersistenceException if retrieving the IDs of the document index fails.
     */
    Set getWordSourceIds(Connection connection, String collectionId)
        throws IndexPersistenceException {
        PreparedStatement pstmt = null;

        try {
            // get the IDs of document indexes belonging to the collection
            pstmt = connection.prepareStatement(SQL_SEL_DOCUMENTS);
            pstmt.setString(1, collectionId);
            return DocumentIndexPersistence.getWordSourceIds(pstmt);
        } catch (SQLException e) {
            throw new IndexPersistenceException("Fails to get word source ids");
        } finally {
            DBUtil.close(pstmt);
        }
    }

    /**
     * <p>
     * Updates the collection index in the persistence. When the collection index ID cannot be found in the
     * persistence, it fails.
     * </p>
     *
     * <p>
     * This method updates records in the DOCUMENT_COLLECTION_DOCUMENT_XREF table. The use count is not updated here,
     * since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionIndex the collection index to be updated.
     *
     * @throws IndexPersistenceException if updating the collection index fails; or the collection index ID cannot be
     *         found in the persistence.
     */
    void updateCollectionIndex(CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        // get the collection id from the collection index
        String collectionId = collectionIndex.getId();

        Connection connection = persistence.getConnection();
        PreparedStatement pstmt = null;
        ResultSet rs = null;

        try {
            // start the transaction
            persistence.beginTransaction(connection);

            // check if the collection index exists
            if (!existCollectionIndex(connection, collectionId)) {
                throw new IndexPersistenceException("Collection index does not exist");
            }

            // extract the document ids from the collection index
            Set newDocumentIds = Util.getDocumentIds(collectionIndex.getAllDocumentIds());
            Set oldDocumentIds = new HashSet();

            // get the document ids from the DOCUMENT_COLLECTION_DOCUMENT_XREF table
            // to determine the document ids to be added and removed
            pstmt = connection.prepareStatement(SQL_SEL_DOCUMENT_IDS);
            pstmt.setString(1, collectionId);
            rs = pstmt.executeQuery();

            while (rs.next()) {
                String documentId = rs.getString(1);

                if (!newDocumentIds.remove(documentId)) {
                    oldDocumentIds.add(documentId);
                }
            }
            rs.close();
            pstmt.close();

            // remove the old documents from the DOCUMENT_COLLECTION_DOCUMENT_XREF table
            DBUtil.execute(connection, SQL_DEL_DOCUMENT, collectionId,
                    (String[]) oldDocumentIds.toArray(new String[0]));

            // add the new documents to the DOCUMENT_COLLECTION_DOCUMENT_XREF table
            DBUtil.execute(connection, SQL_INS_DOCUMENT, collectionId,
                    (String[]) newDocumentIds.toArray(new String[0]));

            // commit the transaction
            persistence.commitTransaction(connection);
        } catch (IndexPersistenceException e) {
            persistence.rollbackTransaction(connection);
            throw e;
        } catch (SQLException e) {
            persistence.rollbackTransaction(connection);
            throw new IndexPersistenceException("Fails to update collection index", e);
        }
    }
}
