/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.persistence.impl.Util;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import java.text.CollationKey;
import java.text.Collator;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * <p>
 * This is an implementation of the AbstractDBIndexPersistence that deals with a simple and straight-forward,
 * normalized database schema. This is provided for a user who is not as concerned about speed of the execution but is
 * more concerned about simple data mining capabilities for 3rd party API. Since all the data is in plain view this
 * can be easily utilized.
 * </p>
 *
 * <p>
 * This class is not thread-safe. The methods should not be called concurrently on a single instance. However, the
 * database operations (add, remove and update) are made atomic using JDBC transactions. So multiple instances of this
 * class can access the database at the same time.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class SimpleAccessDBIndexPersistence extends AbstractDBIndexPersistence {
    /**
     * <p>
     * The SQL statement to insert a word.
     * </p>
     */
    private static final String SQL_INS_WORD = "INSERT INTO WORD VALUES (?, ?, ?)";

    /**
     * <p>
     * The SQL statement to insert a word position.
     * </p>
     */
    private static final String SQL_INS_WORD_POSITION = "INSERT INTO WORD_POSITION VALUES (?, ?, ?)";

    /**
     * <p>
     * The SQL statement to delete the words belonging to a document.
     * </p>
     */
    private static final String SQL_DEL_WORD = "DELETE FROM WORD WHERE DOCUMENT_ID = ?";

    /**
     * <p>
     * The SQL statement to delete the word positions belonging to a document.
     * </p>
     */
    private static final String SQL_DEL_WORD_POSITION = "DELETE FROM WORD_POSITION WHERE DOCUMENT_ID = ?";

    /**
     * <p>
     * The SQL statement to select the words belonging to a document.
     * </p>
     */
    private static final String SQL_SEL_WORDS = "SELECT WORD_ID, WORD_ENTRY FROM WORD WHERE DOCUMENT_ID = ?";

    /**
     * <p>
     * The SQL statement to select the word positions belonging to a document.
     * </p>
     */
    private static final String SQL_SEL_WORD_POSITIONS = "SELECT WORD_POSITION FROM WORD_POSITION "
        + "WHERE DOCUMENT_ID = ? AND WORD_ID = ?";

    /**
     * <p>
     * The underlying persistence providing some common operations related to database persistence of document index.
     * This cannot be changed and cannot be null.
     * </p>
     */
    private final DocumentIndexPersistence documentIndexPersistence = new DocumentIndexPersistence(this);

    /**
     * <p>
     * The underlying persistence providing some common operations related to database persistence of collection index.
     * This cannot be changed and cannot be null.
     * </p>
     */
    private final CollectionIndexPersistence collectionIndexPersistence = new CollectionIndexPersistence(this);

    /**
     * <p>
     * A constructor that loads the connection-related properties from the specified namespace. Simply calls the super
     * constructor.
     * </p>
     *
     * @param namespace The namespace for connection-related properties.
     *
     * @throws IllegalArgumentException if namespace is null or empty string.
     * @throws PersistenceConfigurationException if there are configuration issues encountered.
     */
    public SimpleAccessDBIndexPersistence(String namespace)
        throws PersistenceConfigurationException {
        super(namespace);
    }

    /**
     * <p>
     * A constructor that accepts the given connection factory and connection name. Simply calls the super constructor.
     * If connectionName is null or empty string, the default connection will be used.
     * </p>
     *
     * @param connectionFactory the connection factory used to generate connections.
     * @param connectionName the name of the connection used for persistence operations.
     *
     * @throws IllegalArgumentException if connectionFactory is null.
     */
    public SimpleAccessDBIndexPersistence(DBConnectionFactory connectionFactory, String connectionName) {
        super(connectionFactory, connectionName);
    }

    /**
     * <p>
     * Adds the given document index to the persistence. The initial collection reference count should be 0. When the
     * document index exists in the persistence, it fails.
     * </p>
     *
     * <p>
     * This method adds records to the DOCUMENT, DELIMITER, WORD and WORD_POSITION tables.
     * </p>
     *
     * @param documentIndex the document index to be persisted.
     *
     * @throws IllegalArgumentException if documentIndex is null.
     * @throws IndexPersistenceException if persisting the document index fails; or the document index exists in the
     *         persistence.
     */
    public void addDocumentIndex(DocumentIndex documentIndex)
        throws IndexPersistenceException {
        Util.checkDocumentIndex(documentIndex);

        // get the document id from the word source id
        String documentId = Util.getDocumentId(documentIndex.getWordSourceId());

        Connection connection = getConnection();
        PreparedStatement pstmt = null;
        PreparedStatement pstmt2 = null;

        try {
            // start the transaction
            beginTransaction(connection);

            // add records to the DOCUMENT and DELIMITER tables
            documentIndexPersistence.addDocumentIndex(connection, documentId, documentIndex);

            // perform the specific handling for words and word positions
            Map words = documentIndex.getWords();
            int wordId = 1;

            pstmt = connection.prepareStatement(SQL_INS_WORD);
            pstmt.setString(2, documentId);
            pstmt2 = connection.prepareStatement(SQL_INS_WORD_POSITION);
            pstmt2.setString(1, documentId);

            for (Iterator i = words.entrySet().iterator(); i.hasNext(); wordId++) {
                // extract each word and its positions
                Map.Entry entry = (Map.Entry) i.next();
                String word = ((CollationKey) entry.getKey()).getSourceString();
                List positions = (List) entry.getValue();

                // add the word to the WORD table
                pstmt.setInt(1, wordId);
                pstmt.setString(3, word);
                pstmt.executeUpdate();

                // add the word positions to the WORD_POSITION table
                pstmt2.setInt(2, wordId);
                for (Iterator j = positions.iterator(); j.hasNext();) {
                    pstmt2.setInt(3, ((Integer) j.next()).intValue());
                    pstmt2.executeUpdate();
                }
            }
            pstmt.close();
            pstmt2.close();

            // commit the transaction
            commitTransaction(connection);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(connection);
            throw e;
        } catch (SQLException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Fails to add document index", e);
        }
    }

    /**
     * <p>
     * Gets the document index from the persistence. The ID of the document is given. When the document is not
     * persisted in the persistence, null should be returned.
     * </p>
     *
     * <p>
     * This method retrieves records from the WORD and WORD_POSITION tables. It does not deal with the DOCUMENT and
     * DELIMITER tables since the given wordSourceId already provides the information.
     * </p>
     *
     * @param wordSourceId the ID of the document whose index is retrieved.
     *
     * @return the document index of the document with the ID, or null if not found.
     *
     * @throws IllegalArgumentException if wordSourceId is null.
     * @throws IndexPersistenceException if retrieving the document index fails.
     */
    public DocumentIndex getDocumentIndex(WordSourceId wordSourceId)
        throws IndexPersistenceException {
        Util.checkWordSourceId(wordSourceId);

        // get the document id from the word source id
        String documentId = Util.getDocumentId(wordSourceId);

        // prepare the collator under the current locale
        Collator collator = Collator.getInstance(wordSourceId.getSourceLocale());

        Connection connection = getConnection();
        PreparedStatement pstmt = null;
        ResultSet rs = null;

        try {
            // check if the document index exists
            if (!documentIndexPersistence.existDocumentIndex(connection, documentId)) {
                return null;
            }

            // get the words from the WORD table
            pstmt = connection.prepareStatement(SQL_SEL_WORDS);
            pstmt.setString(1, documentId);
            rs = pstmt.executeQuery();

            // save the mapping from words to word ids,
            // then free up the resources immediately since the number of words is large
            Map wordIds = new HashMap();

            while (rs.next()) {
                wordIds.put(rs.getString(2), new Integer(rs.getInt(1)));
            }
            rs.close();
            pstmt.close();

            // get the word positions from the WORD_POSITION table
            pstmt = connection.prepareStatement(SQL_SEL_WORD_POSITIONS);
            pstmt.setString(1, documentId);

            // prepare the mapping from collation keys to word positions
            Map words = new HashMap();

            for (Iterator i = wordIds.entrySet().iterator(); i.hasNext();) {
                // extract each word and its word id
                Map.Entry entry = (Map.Entry) i.next();
                String word = (String) entry.getKey();
                int wordId = ((Integer) entry.getValue()).intValue();

                pstmt.setInt(2, wordId);
                rs = pstmt.executeQuery();

                // add the list of word positions to the map
                words.put(collator.getCollationKey(word), DBUtil.getIntegers(rs));

                // free up the resources immediately since the number of word positions is large
                rs.close();
            }
            return new DocumentIndex(wordSourceId, words);
        } catch (SQLException e) {
            throw new IndexPersistenceException("Fails to get document index", e);
        } finally {
            DBUtil.close(rs);
            DBUtil.close(pstmt);
            DBUtil.close(connection);
        }
    }

    /**
     * <p>
     * Removes a document index from the persistence. The document index can be deleted only when there is no more
     * collection containing the document index. The ID of the document is given. When the document is not persisted,
     * it fails.
     * </p>
     *
     * <p>
     * This method deletes records from the DOCUMENT, DELIMITER, WORD and WORD_POSITION tables.
     * </p>
     *
     * @param wordSourceId the ID of the document whose index is deleted.
     *
     * @throws IllegalArgumentException if wordSourceId is null.
     * @throws IndexPersistenceException if deleting the document index fails; or the document index is not persisted;
     *         or there is at least one collection containing the document index.
     */
    public void removeDocumentIndex(WordSourceId wordSourceId)
        throws IndexPersistenceException {
        Util.checkWordSourceId(wordSourceId);

        // get the document id from the word source id
        String documentId = Util.getDocumentId(wordSourceId);

        Connection connection = getConnection();

        try {
            // start the transaction
            beginTransaction(connection);

            // check if the documnt index exists and is unused
            if (!documentIndexPersistence.existUnusedDocumentIndex(connection, documentId)) {
                throw new IndexPersistenceException("Unused document index does not exist");
            }

            // remove the word positions from the WORD_POSITION table
            DBUtil.execute(connection, SQL_DEL_WORD_POSITION, documentId);

            // remove the words from the WORD table
            DBUtil.execute(connection, SQL_DEL_WORD, documentId);

            // remove records from the DOCUMENT and DELIMITER tables
            documentIndexPersistence.removeDocumentIndex(connection, documentId);

            // commit the transaction
            commitTransaction(connection);
        } catch (IndexPersistenceException e) {
            rollbackTransaction(connection);
            throw e;
        } catch (SQLException e) {
            rollbackTransaction(connection);
            throw new IndexPersistenceException("Fails to remove document index", e);
        }
    }

    /**
     * <p>
     * Adds the collection index to the persistence. When the collection index exists in the persistence, it fails.
     * When the document indexes of the collection index are not persisted, it also fails.
     * </p>
     *
     * <p>
     * This method adds records to the DOCUMENT_COLLECTION and DOCUMENT_COLLECTION_DOCUMENT_XREF tables. The use count
     * is not updated here, since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionIndex the collection index to be persisted.
     *
     * @throws IllegalArgumentException if collectionIndex is null.
     * @throws IndexPersistenceException if persisting the collection index fails; or the collection index exists in
     *         the persistence; or the document indexes of the collection index are not persisted.
     */
    public void addCollectionIndex(CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        Util.checkCollectionIndex(collectionIndex);
        collectionIndexPersistence.addCollectionIndex(collectionIndex);
    }

    /**
     * <p>
     * Gets the collection index from the persistence. The ID of the collection index is given. When the collection
     * index is not persisted, null should be returned.
     * </p>
     *
     * <p>
     * This method retrieves records from the DOCUMENT_COLLECTION, DOCUMENT_COLLECTION_DOCUMENT_XREF, DOCUMENT,
     * DELIMITER and WORD tables. Note that the collection index is created with null owner. It should be set by the
     * IndexManager using the setOwner() method.
     * </p>
     *
     * @param collectionId the ID of the collection index to be retrieved.
     *
     * @return the collection index with the given ID.
     *
     * @throws IllegalArgumentException if collectionId is null or empty string.
     * @throws IndexPersistenceException if retrieving the collection index fails.
     */
    public CollectionIndex getCollectionIndex(String collectionId)
        throws IndexPersistenceException {
        Util.checkCollectionId(collectionId);

        Connection connection = getConnection();
        PreparedStatement pstmt = null;
        ResultSet rs = null;

        try {
            // check if the collection index exists
            if (!collectionIndexPersistence.existCollectionIndex(connection, collectionId)) {
                return null;
            }

            // get the document IDs in the collection
            Set wordSourceIds = collectionIndexPersistence.getWordSourceIds(connection, collectionId);
            Map words = new HashMap();

            // perform the specific handling for words
            pstmt = connection.prepareStatement(SQL_SEL_WORDS);

            for (Iterator i = wordSourceIds.iterator(); i.hasNext();) {
                // extract each word source id and its collator
                WordSourceId wordSourceId = (WordSourceId) i.next();
                String documentId = Util.getDocumentId(wordSourceId);
                Collator collator = Collator.getInstance(wordSourceId.getSourceLocale());

                // get the words from the WORD table
                // then free up the resources immediately since the number of words is large
                pstmt.setString(1, documentId);
                rs = pstmt.executeQuery();

                while (rs.next()) {
                    CollationKey key = collator.getCollationKey(rs.getString(2));
                    Set value = (Set) words.get(key);

                    // if the word has not been added, add a new set to the map
                    if (value == null) {
                        value = new HashSet();
                        words.put(key, value);
                    }

                    // add the word source id to the set
                    value.add(wordSourceId);
                }
                rs.close();
            }

            // create the collection index with null owner
            // it should be set by the IndexManager using the setOwner() method
            return new CollectionIndex(null, words, wordSourceIds, collectionId);
        } catch (SQLException e) {
            throw new IndexPersistenceException("Fails to get collection index", e);
        } finally {
            DBUtil.close(rs);
            DBUtil.close(pstmt);
            DBUtil.close(connection);
        }
    }

    /**
     * <p>
     * Removes the collection index from the persistence. The ID of the collection index is given. When the collection
     * index is not persisted, it fails.
     * </p>
     *
     * <p>
     * This method deletes records from the DOCUMENT_COLLECTION and DOCUMENT_COLLECTION_DOCUMENT_XREF tables. The use
     * count is not updated here, since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionId the ID of the collection index to be deleted.
     *
     * @throws IllegalArgumentException if collectionId is null or empty string.
     * @throws IndexPersistenceException if deleting the collection index fails; or the collection index ID cannot be
     *         found in the persistence.
     */
    public void removeCollectionIndex(String collectionId)
        throws IndexPersistenceException {
        Util.checkCollectionId(collectionId);
        collectionIndexPersistence.removeCollectionIndex(collectionId);
    }

    /**
     * <p>
     * Updates the collection index in the persistence. When the collection index ID cannot be found in the
     * persistence, it fails.
     * </p>
     *
     * <p>
     * This method updates records in the DOCUMENT_COLLECTION_DOCUMENT_XREF table. The use count is not updated here,
     * since it is handled by the CollectionIndex class.
     * </p>
     *
     * @param collectionIndex the collection index to be updated.
     *
     * @throws IllegalArgumentException if collectionIndex is null.
     * @throws IndexPersistenceException if updating the collection index fails; or the collection index ID cannot be
     *         found in the persistence.
     */
    public void updateCollectionIndex(CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        Util.checkCollectionIndex(collectionIndex);
        collectionIndexPersistence.updateCollectionIndex(collectionIndex);
    }

    /**
     * <p>
     * Increases the collection reference count of the document index. The document ID is given. When the document
     * index is not persisted, it fails.
     * </p>
     *
     * <p>
     * This method updates a record in the DOCUMENT table.
     * </p>
     *
     * @param wordSourceId the ID of the document whose collection reference count is increased by 1.
     *
     * @throws IllegalArgumentException if wordSourceId is null.
     * @throws IndexPersistenceException if increasing the collection reference count fails; or the document index is
     *         not persisted.
     */
    public void increaseDocumentUseCount(WordSourceId wordSourceId)
        throws IndexPersistenceException {
        Util.checkWordSourceId(wordSourceId);
        documentIndexPersistence.increaseDocumentUseCount(wordSourceId);
    }

    /**
     * <p>
     * Decreases the collection reference count of the document index. The document ID is given. When the document
     * index is not persisted, it fails.
     * </p>
     *
     * <p>
     * This method updates a record in the DOCUMENT table.
     * </p>
     *
     * @param wordSourceId the ID of the document whose collection reference count is decreased by 1.
     *
     * @throws IllegalArgumentException if wordSourceId is null.
     * @throws IndexPersistenceException if decreasing the collection reference count fails; or the document index is
     *         not persisted.
     */
    public void decreaseDocumentUseCount(WordSourceId wordSourceId)
        throws IndexPersistenceException {
        Util.checkWordSourceId(wordSourceId);
        documentIndexPersistence.decreaseDocumentUseCount(wordSourceId);
    }

    /**
     * <p>
     * Gets a set of all IDs of documents whose indexes are persisted in the persistence.
     * </p>
     *
     * <p>
     * This method retrieves records from the DOCUMENT and DELIMITER tables.
     * </p>
     *
     * @return a set of all IDs of persisted document indexes.
     *
     * @throws IndexPersistenceException if retrieving the document IDs fails.
     */
    public Set getIndexedDocuments() throws IndexPersistenceException {
        return documentIndexPersistence.getIndexedDocuments();
    }

    /**
     * <p>
     * Gets the collection reference count of the given document index. When the document index is not persisted, it
     * fails.
     * </p>
     *
     * <p>
     * This method retrieves a record from the DOCUMENT table.
     * </p>
     *
     * @param wordSourceId the ID of the document index whose collection reference count is retrieved.
     *
     * @return the reference count of the document index.
     *
     * @throws IllegalArgumentException if wordSourceId is null.
     * @throws IndexPersistenceException if retrieving the collection reference count fails; or the document index is
     *         not persisted.
     */
    public int getDocumentUseCount(WordSourceId wordSourceId)
        throws IndexPersistenceException {
        Util.checkWordSourceId(wordSourceId);
        return documentIndexPersistence.getDocumentUseCount(wordSourceId);
    }
}
