/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import java.util.HashSet;
import java.util.Set;


/**
 * <p>
 * A parser to extract information in the collection index file using SAX. The extracted information includes the
 * document ids.
 * </p>
 *
 * <p>
 * Note that the parser does not verify the consistency of the document indexes with the collection indexes, as well as
 * the structure of the xml file. The contents of the files are assumed to be maintained by the XmlIndexPersistence
 * class, so consistency is assured.
 * </p>
 *
 * <p>
 * This class is package-private, so that they can only be used by classes in the same package.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
class CollectionIndexParser extends DefaultHandler {
    /**
     * <p>
     * The set of document ids (String) contained in the collection index. This cannot be changed. The ids are
     * populated once the xml file is parsed.
     * </p>
     */
    private final Set documentIds = new HashSet();

    /**
     * <p>
     * A boolean indicating whether the next character data represents the document id. This is initialized to false.
     * It can be changed when the xml file is parsed.
     * </p>
     */
    private boolean parseDocumentId = false;

    /**
     * <p>
     * Creates a new instance of CollectionIndexParser
     * </p>
     */
    public CollectionIndexParser() {
    }

    /**
     * <p>
     * Determines when the document id starts.
     * </p>
     *
     * @param uri the uri namespace.
     * @param localName the local name (without prefix).
     * @param qName the qualified name (with prefix).
     * @param attributes the specified attributes.
     */
    public void startElement(String uri, String localName, String qName, Attributes attributes) {
        if (qName.equals(XmlConstants.DOCUMENT_ID_TAG_NAME)) {
            parseDocumentId = true;
        }
    }

    /**
     * <p>
     * Determines when the document id ends.
     * </p>
     *
     * @param uri the uri namespace.
     * @param localName the local name (without prefix).
     * @param qName the qualified name (with prefix).
     */
    public void endElement(String uri, String localName, String qName) {
        if (qName.equals(XmlConstants.DOCUMENT_ID_TAG_NAME)) {
            parseDocumentId = false;
        }
    }

    /**
     * <p>
     * Parses the document id information.
     * </p>
     *
     * @param ch the character array.
     * @param start the start position in the character array.
     * @param length the number of characters to use from the character array.
     */
    public void characters(char[] ch, int start, int length) {
        if (parseDocumentId) {
            // add the document id to the set
            documentIds.add(new String(ch, start, length));
        }
    }

    /**
     * <p>
     * Returns the set of all document ids in the collection index file. A copy of the inner set is returned.
     * </p>
     *
     * <p>
     * This method should be called after the xml file is parsed.
     * </p>
     *
     * @return the set of all document ids.
     */
    public Set getAllDocumentIds() {
        return new HashSet(documentIds);
    }
}
