/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.persistence.IndexPersistenceException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;


/**
 * <p>
 * A utility class providing some common operations related to xml file. This class and all its methods are
 * package-private, so that they can only be used by classes in the same package.
 * </p>
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
class XmlUtil {
    /**
     * <p>
     * Private constructor to prevent outside instantiation.
     * </p>
     */
    private XmlUtil() {
    }

    /**
     * <p>
     * Parses the given file using SAX. The specified handler is used to extract information from the file.
     * </p>
     *
     * @param file the xml file to parse.
     * @param handler the SAX handler to use.
     *
     * @throws IndexPersistenceException if any error occurs during parsing.
     */
    static void parse(File file, DefaultHandler handler)
        throws IndexPersistenceException {
        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();

            parser.parse(file, handler);
        } catch (ParserConfigurationException e) {
            throw new IndexPersistenceException("SAX parsing is not supported", e);
        } catch (SAXException e) {
            throw new IndexPersistenceException("Fails to parse the file", e);
        } catch (IOException e) {
            throw new IndexPersistenceException("Some I/O error occurs", e);
        }
    }

    /**
     * <p>
     * Parses the given file using DOM.
     * </p>
     *
     * @param file the xml file to parse.
     *
     * @return the DOM document object.
     *
     * @throws IndexPersistenceException if any error occurs during parsing.
     */
    static Document parse(File file) throws IndexPersistenceException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();

            return builder.parse(file);
        } catch (ParserConfigurationException e) {
            throw new IndexPersistenceException("DOM parsing is not supported", e);
        } catch (SAXException e) {
            throw new IndexPersistenceException("Fails to parse from the file", e);
        } catch (IOException e) {
            throw new IndexPersistenceException("Some I/O error occurs", e);
        }
    }

    /**
     * <p>
     * Transforms the DOM document object into an xml file. The output xml is not pretty formatted.
     * </p>
     *
     * @param file the xml file to transform to.
     * @param document the DOM document object.
     *
     * @throws IndexPersistenceException if any error occurs during transformation.
     */
    static void transform(File file, Document document)
        throws IndexPersistenceException {
        try {
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();

            transformer.transform(new DOMSource(document), new StreamResult(file));
        } catch (TransformerConfigurationException e) {
            throw new IndexPersistenceException("DOM transformation is not supported", e);
        } catch (TransformerException e) {
            throw new IndexPersistenceException("Fails to write to the file", e);
        }
    }

    /**
     * <p>
     * Converts the given node list to an array of nodes.
     * </p>
     *
     * @param list the node list to convert.
     *
     * @return an array of nodes.
     */
    static Node[] toNodeArray(NodeList list) {
        Node[] nodes = new Node[list.getLength()];

        for (int i = 0; i < nodes.length; i++) {
            nodes[i] = list.item(i);
        }
        return nodes;
    }
}
