/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.DBConnectionFactory;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.ConfigHelper;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.sql.Connection;
import java.sql.PreparedStatement;

import java.util.Set;


/**
 * Unit tests for AbstractDBIndexPersistence implementation.
 *
 * <p>
 * We assume here the instance loaded from configuration namespace will behave the same as created with two-argument
 * constructor. So only the former is used for testing.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AbstractDBIndexPersistenceTest extends TestCase {
    /**
     * The sql query for testing transactions.
     */
    private static final String SQL_QUERY = "INSERT INTO DOCUMENT_COLLECTION VALUES (1)";

    /**
     * The AbstractDBIndexPersistence instance to test against.
     */
    private AbstractDBIndexPersistence persistence = null;

    /**
     * The connection object obtained from the AbstractDBIndexPersistence instance.
     */
    private Connection connection = null;

    /**
     * The prepared statement to be executed.
     */
    private PreparedStatement pstmt = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(AbstractDBIndexPersistenceTest.class);

        return suite;
    }

    /**
     * Prepares an AbstractDBIndexPersistence instance for testing.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        ConfigHelper.loadConfig();
        DBHelper.clearTables();
        persistence = new AbstractDBIndexPersistenceImpl(ConfigHelper.DB_NAMESPACE);
        connection = persistence.getConnection();
    }

    /**
     * Clears all the namespaces and database resources.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void tearDown() throws Exception {
        DBUtil.close(pstmt);
        DBUtil.close(connection);
        DBHelper.clearTables();
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null namespace. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullNamespace() throws Exception {
        try {
            new AbstractDBIndexPersistenceImpl(null);
            fail("Creates AbstractDBIndexPersistence with null namespace");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty namespace. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyNamespace() throws Exception {
        try {
            new AbstractDBIndexPersistenceImpl("");
            fail("Creates AbstractDBIndexPersistence with empty namespace");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad namespaces. Expects PersistenceConfigurationException.
     */
    public void testConstructor_BadNamespaces() {
        String[] namespaces = ConfigHelper.BAD_DB_NAMESPACES;

        for (int i = 0; i < namespaces.length; i++) {
            try {
                new AbstractDBIndexPersistenceImpl(namespaces[i]);
                fail("Creates AbstractDBIndexPersistence with bad namespace");
            } catch (PersistenceConfigurationException e) {
                // good
            }
        }
    }

    /**
     * Test of constructor with null connection factory. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullConnectionFactory()
        throws Exception {
        try {
            new AbstractDBIndexPersistenceImpl(null, ConfigHelper.CONNECTION_PRODUCER_NAME);
            fail("Creates AbstractDBIndexPersistence with null connection factory");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getConnection method. Verifies if the returned connection is opened, and have the auto commit set to
     * false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetConnection() throws Exception {
        assertFalse("Fails to open the connection", connection.isClosed());
        assertFalse("Fails to disable auto commit", connection.getAutoCommit());
    }

    /**
     * Test of beginTransaction method with null connection. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testBeginTransaction_NullConnection() throws Exception {
        try {
            persistence.beginTransaction(null);
            fail("Begins transaction with null connection");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of beginTransaction method with non-null connection. Verifies if the auto commit is set to false after
     * calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testBeginTransaction_NonNullConnection()
        throws Exception {
        persistence.beginTransaction(connection);
        assertFalse("Fails to disable auto commit", connection.getAutoCommit());
    }

    /**
     * Test of commitTransaction method with null connection. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testCommitTransaction_NullConnection()
        throws Exception {
        try {
            persistence.commitTransaction(null);
            fail("Commits transaction with null connection");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of commitTransaction method with non-null connection. Executes a sql query to add a record. Verifies if the
     * record was added to the persistence after calling it. Also verifies if the connection is closed.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testCommitTransaction_NonNullConnection()
        throws Exception {
        pstmt = connection.prepareStatement(SQL_QUERY);
        pstmt.executeUpdate();
        persistence.commitTransaction(connection);
        assertFalse("Fails to save database changes", DBHelper.isEmptyTable("DOCUMENT_COLLECTION"));
        assertTrue("Fails to close the connection", connection.isClosed());
    }

    /**
     * Test of rollbackTransaction method with null connection. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testRollbackTransaction_NullConnection()
        throws Exception {
        try {
            persistence.rollbackTransaction(null);
            fail("Rolls back transaction with null connection");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of rollbackTransaction method with non-null connection. Executes a sql query to add a record. Verifies if
     * the record was not added to the persistence after calling it. Also verifies if the connection is closed.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testRollbackTransaction_NonNullConnection()
        throws Exception {
        pstmt = connection.prepareStatement(SQL_QUERY);
        pstmt.executeUpdate();
        persistence.rollbackTransaction(connection);
        assertTrue("Fails to save database changes", DBHelper.isEmptyTable("DOCUMENT_COLLECTION"));
        assertTrue("Fails to close the connection", connection.isClosed());
    }

    /**
     * Simple implementation of AbstractDBIndexPersistence for testing purpose.
     */
    private class AbstractDBIndexPersistenceImpl extends AbstractDBIndexPersistence {
        /**
         * A constructor that loads the connection-related properties from the specified namespace.
         *
         * @param namespace The namespace for connection-related properties.
         *
         * @throws IllegalArgumentException if namespace is null or empty string.
         * @throws PersistenceConfigurationException if there are configuration issues encountered.
         */
        public AbstractDBIndexPersistenceImpl(String namespace)
            throws PersistenceConfigurationException {
            super(namespace);
        }

        /**
         * A constructor that accepts the given connection factory and connection name.
         *
         * @param connectionFactory the connection factory used to generate connections.
         * @param connectionName the name of the connection used for persistence operations.
         *
         * @throws IllegalArgumentException if connectionFactory is null.
         */
        public AbstractDBIndexPersistenceImpl(DBConnectionFactory connectionFactory, String connectionName) {
            super(connectionFactory, connectionName);
        }

        /**
         * Empty method implementation.
         *
         * @param documentIndex the document index argument.
         */
        public void addDocumentIndex(DocumentIndex documentIndex) {
        }

        /**
         * Empty method implementation.
         *
         * @param wordSourceId the word source id argument.
         *
         * @return always null.
         */
        public DocumentIndex getDocumentIndex(WordSourceId wordSourceId) {
            return null;
        }

        /**
         * Empty method implementation.
         *
         * @param wordSourceId the word source id argument.
         */
        public void removeDocumentIndex(WordSourceId wordSourceId) {
        }

        /**
         * Empty method implementation.
         *
         * @param collectionIndex the collection index argument.
         */
        public void addCollectionIndex(CollectionIndex collectionIndex) {
        }

        /**
         * Empty method implementation.
         *
         * @param collectionId the collection id argument.
         *
         * @return always null.
         */
        public CollectionIndex getCollectionIndex(String collectionId) {
            return null;
        }

        /**
         * Empty method implementation.
         *
         * @param collectionId the collection id argument.
         */
        public void removeCollectionIndex(String collectionId) {
        }

        /**
         * Empty method implementation.
         *
         * @param collectionIndex the collection index argument.
         */
        public void updateCollectionIndex(CollectionIndex collectionIndex) {
        }

        /**
         * Empty method implementation.
         *
         * @param wordSourceId the word source id argument.
         */
        public void increaseDocumentUseCount(WordSourceId wordSourceId) {
        }

        /**
         * Empty method implementation.
         *
         * @param wordSourceId the word source id argument.
         */
        public void decreaseDocumentUseCount(WordSourceId wordSourceId) {
        }

        /**
         * Empty method implementation.
         *
         * @param wordSourceId the word source id argument.
         *
         * @return always 0.
         */
        public int getDocumentUseCount(WordSourceId wordSourceId) {
            return 0;
        }

        /**
         * Empty method implementation.
         *
         * @return always null.
         */
        public Set getIndexedDocuments() {
            return null;
        }
    }
}
