/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Writer;


/**
 * A helper class to perform those xml operations for testing. These include setting up and cleaning up of persistence
 * directory, and comparing the contents of the files in two directories.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlHelper {
    /**
     * The base directory path where the expected index files are stored (in sub-directory).
     */
    public static final String EXPECTED_DIR_BASE_PATH = "test_files/";

    /**
     * The working directory path where the index files are stored.
     */
    public static final String PERSISTENCE_DIR_PATH = "test_files/working/";

    /**
     * The name of the document usage file.
     */
    public static final String DOCUMENT_USAGE_FILE = "document_usage.xml";

    /**
     * Private constructor to prevent instantiation.
     */
    private XmlHelper() {
    }

    /**
     * Verifies if the contents of the files in the working directory are the same as the given expected directory.
     * Besides, if one file exists in a directory but not in another, false will be returned.
     *
     * @param expectedDirPath the name of the directory for the expected files.
     *
     * @return whether the files in the working directory are expected.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public static boolean verify(String expectedDirPath)
        throws Exception {
        File[] files1 = new File(EXPECTED_DIR_BASE_PATH, expectedDirPath).listFiles();
        File[] files2 = new File(PERSISTENCE_DIR_PATH).listFiles();

        // check the number of files
        if (files1.length != files2.length) {
            return false;
        }

        for (int i = 0; i < files1.length; i++) {
            File file = new File(PERSISTENCE_DIR_PATH, files1[i].getName());

            // check if the file of the expected name exists
            if (!file.exists()) {
                return false;
            }

            // verify the contents of the xml files
            if (!verifyXml(files1[i], file)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Verifies if the generated xml specified in the xmlFile2 is exactly the same as the correct xml stored in the
     * xmlFile1. The comparison is line-by-line based, so that the line characters would not cause a problem in
     * different OS.
     *
     * @param xmlFile1 the file storing the correct xml.
     * @param xmlFile2 the file holding the generated xml.
     *
     * @return whether the generated xml is correct.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    private static boolean verifyXml(File xmlFile1, File xmlFile2)
        throws Exception {
        BufferedReader reader1 = new BufferedReader(new FileReader(xmlFile1));
        BufferedReader reader2 = new BufferedReader(new FileReader(xmlFile2));

        return verifyXml(reader1, reader2);
    }

    /**
     * Verifies if the xmls pointed by the two readers are the same. The comparison is line-by-line based, so that the
     * line characters would not cause a problem in different OS.
     *
     * @param reader1 the first reader.
     * @param reader2 the second reader.
     *
     * @return whether the contents of the readers are the same.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    private static boolean verifyXml(BufferedReader reader1, BufferedReader reader2)
        throws Exception {
        String line1 = null;
        String line2 = null;

        while (true) {
            line1 = reader1.readLine();
            line2 = reader2.readLine();

            if ((line1 == null) || (line2 == null)) {
                break; // stop if any reader reaches the end
            }
            if (!line1.equals(line2)) {
                break; // stop if there is any difference
            }
        }
        reader1.close();
        reader2.close();

        // both readers should reach the end
        return (line1 == null) && (line2 == null);
    }

    /**
     * <p>
     * Sets up the working directory for xml persistence. Basically it writes one document usage file (containing
     * documents tag) in the directory.
     * </p>
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public static void setupDir() throws Exception {
        cleanupDir();

        Writer writer = new BufferedWriter(new FileWriter(new File(PERSISTENCE_DIR_PATH, DOCUMENT_USAGE_FILE)));

        writer.write("<documents></documents>");
        writer.close();
    }

    /**
     * Deletes all files in the working directory.
     */
    public static void cleanupDir() {
        File[] files = new File(PERSISTENCE_DIR_PATH).listFiles();

        for (int i = 0; i < files.length; i++) {
            files[i].delete();
        }
    }
}
